package com.zx.dataservice.controller;

/**
 * Created by qlm on 2020-12-14.
 */

import com.alibaba.fastjson.JSON;
import com.zx.dataservice.options.WSOptions;
import com.zx.dataservice.utils.DateUtils;
import com.zx.dataservice.vo.StockRestRedisVO;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Component;

import javax.websocket.*;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.concurrent.ConcurrentHashMap;
import java.lang.reflect.Field;
import java.net.InetSocketAddress;
import javax.websocket.RemoteEndpoint.Async;

@ServerEndpoint("/websocket/{userId}")
@Component
public class WebSocketController {

//    public static Map<String,Session> map = new HashMap<String,Session>();

    private static ConcurrentHashMap<String, Session> map = new ConcurrentHashMap<>();//根据用户找session


    public static InetSocketAddress getRemoteAddress(Session session) {
        if (session == null) {
            return null;
        }
        Async async = session.getAsyncRemote();

        //在Tomcat 8.0.x版本有效
//		InetSocketAddress addr = (InetSocketAddress) getFieldInstance(async,"base#sos#socketWrapper#socket#sc#remoteAddress");
        //在Tomcat 8.5以上版本有效
        InetSocketAddress addr = (InetSocketAddress) getFieldInstance(async,"base#socketWrapper#socket#sc#remoteAddress");
        return addr;
    }


    private static Object getFieldInstance(Object obj, String fieldPath) {
        String fields[] = fieldPath.split("#");
        for (String field : fields) {
            obj = getField(obj, obj.getClass(), field);
            if (obj == null) {
                return null;
            }
        }

        return obj;
    }

    private static Object getField(Object obj, Class<?> clazz, String fieldName) {
        for (; clazz != Object.class; clazz = clazz.getSuperclass()) {
            try {
                Field field;
                field = clazz.getDeclaredField(fieldName);
                field.setAccessible(true);
                return field.get(obj);
            } catch (Exception e) {
            }
        }

        return null;
    }

    /**
     * 连接建立成功时调用
     * @param userId 登录用户id
     * @param session  可选的参数。session为与某个客户端的连接会话，需要通过它来给客户端发送数据
     * @throws IOException
     */
    @OnOpen
    public void onOpen(Session session, @PathParam("userId") String userId) throws IOException{
        String[] a = userId.split("&");
        String mUserId = a[0];
        map.put(mUserId + "",session);       //添加到链接map
        try{
            InetSocketAddress remoteAddress = getRemoteAddress(session);
            System.out.println(remoteAddress+DateUtils.getCurrentTimeSSS() +mUserId+"连接成功");
            sendUnreadMsg(session);
        }
        catch (Exception e){
            e.printStackTrace();
        }
    }

    /**
     * 连接成功时发送的信息
     */
    public static void sendUnreadMsg(Session session){
            synchronized(session){
                if(session.isOpen()){
                    try {
                        session.getBasicRemote().sendText("testws"+DateUtils.getCurrentTimeSSS()+"连接成功");
                    }catch (Exception e) {
                        e.printStackTrace();
                    }
                }
            }
    }


    /**
     * 连接关闭调用的方法
     */
    @OnClose
    public void onClose(Session session){
        if(map.containsValue(session)){
            for (Entry<String, Session> entry : map.entrySet()){
                if(entry.getValue().equals(session)){
                    map.remove(entry.getKey());
                    break;
                }
            }
        }
    }

    /**
     * 发生错误时调用
     * @param session
     * @param error
     */
    @OnError
    public void onError(Session session, Throwable error){
        if(map.containsValue(session)){
            for (Entry<String, Session> entry : map.entrySet()) {
                if(entry.getValue().equals(session)){
                    map.remove(entry.getKey());
                    break;
                }
            }
        }
        error.printStackTrace();
    }

    public void remWSKey(String key){
        map.remove(key);
    }

    /**
     * 消息发送时调用
     * @param requestJson
     * @param session
     */
    @OnMessage
    public void onMessage(String requestJson, Session session) {
        System.out.println(requestJson);
        if("close".equals(requestJson)){
            onClose(session);
        }

    }


    /**
     */
    public static void sendSysMsg(String id ,String  xMessage){

        try{
            // 数据服务器 id   ：
            Session session = map.get( id);               //发送消息给对方
            if(session!=null&&session.isOpen()){
                synchronized(session){
                    if(session.isOpen()){
                        session.getBasicRemote().sendText(xMessage);
                    }
                }
            }
        }catch(IOException e) {
            e.printStackTrace();
        }
    }
    /**
     *根据订阅发送消息
     */
    public static void sendSysMsgType(String type ,Map<String,List<StockRestRedisVO>>mapws ){
        try{
            for(String key:map.keySet()){
                Session session = map.get(key);
                if(session!=null&&session.isOpen()&& StringUtils.isNotBlank(WSOptions.get(key))){
                    if(WSOptions.get(key).split("_")[1].toLowerCase().equals("ALL".toLowerCase())||WSOptions.get(key).split("_")[1].toLowerCase().equals(type.toLowerCase())){ // 推送所有
                        // 推送对应的 风控 记录
                        synchronized(session){
                            if(session.isOpen()){
                                List<StockRestRedisVO> stockRestRedisVOS = mapws.get("stockRestRedisVO_" + WSOptions.get(key).split("_")[0]);
                                session.getBasicRemote().sendText(JSON.toJSONString(stockRestRedisVOS));
                            }
                        }
                    }
                }else { // 如果对方不是自己人 就删除
                    map.remove(key);
                }
            }
        }catch(IOException e) {
            e.printStackTrace();
        }
    }

    public static void soutLog(){
        for(String key:map.keySet()){
            Session session = map.get(key);
            String a = "";
            if(session!=null&&session.isOpen()){
                a =  "还在还在";
            }else {
                a = "已经挂掉了不在了" ;
                map.remove(key);
            }
            System.out.println(DateUtils.getCurrentTimeSSS()+"存在的连接有*"+key+"***"+a);
        }
    }

    /**
     *根据订阅发送消息
     */
    public static void sendSysMsgType(String type ,String  xMessage ){
        try{
            for(String key:map.keySet()){
                Session session = map.get(key);
                // 每个小时打印 还在的 连接
                if(session!=null&&session.isOpen()&& StringUtils.isNotBlank(WSOptions.get(key))){
                    if(WSOptions.get(key).split("_")[1].toLowerCase().equals("ALL".toLowerCase())||WSOptions.get(key).split("_")[1].toLowerCase().equals(type.toLowerCase())){ // 推送所有
                        // 推送对应的 风控 记录
                        synchronized(session){
                            if(session.isOpen()){
                                session.getBasicRemote().sendText(xMessage);
                            }
                        }
                    }
                }else { // 如果对方不是自己人 就删除
                    map.remove(key);
                }
            }
        }catch(IOException e) {
            e.printStackTrace();
        }
    }

}