package com.zx.dataservice.task;

import com.zx.dataservice.service.ChoiceStockService;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.scheduling.annotation.Async;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

/**
 * 解析东方财富实时数据
 */
@Component
public class AnalysisStockRestDataTask {

    private static final Logger log = LoggerFactory.getLogger(AnalysisStockRestDataTask.class);

    @Autowired
    private ChoiceStockService choiceStockService;

    @Async("executorStock")
    @Scheduled(fixedRate=10)
    public void toAnalysisRestData() {
        doTask("now");
    }

    public void doTask(String type) {
        this.choiceStockService.analysisStockRestData(type);
    }

    @Async("executorAfterStock")
    @Scheduled(cron = "0/2 * 15,16,17,18,19 * * ? ")
    public void toAnalysisRestAfterData() {
        this.choiceStockService.analysisStockRestDataAfter("after");
    }

    @Async("executorInFrontStock")
    @Scheduled(cron = "0/5 1-30 9 * * ? ")
    public void toAnalysisRestInFrontData() {
        this.choiceStockService.analysisStockRestDataInFront("inFront");
    }


}
